/*
 * Copyright (C) 2013 Red Hat Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *     * Redistributions of source code must retain the above
 *       copyright notice, this list of conditions and the
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the
 *       above copyright notice, this list of conditions and
 *       the following disclaimer in the documentation and/or
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be
 *       used to endorse or promote products derived from this
 *       software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 *
 * Author: Stef Walter <stefw@redhat.com>
 */

#ifndef P11_PERSIST_H_
#define P11_PERSIST_H_

#include "array.h"
#include "buffer.h"
#include "compat.h"
#include "dict.h"
#include "pkcs11.h"

#include <sys/types.h>

typedef struct _p11_persist p11_persist;

p11_persist *    p11_persist_new    (void);

bool             p11_persist_magic  (const unsigned char *data,
                                     size_t length);

bool             p11_persist_read   (p11_persist *persist,
                                     const char *filename,
                                     const unsigned char *data,
                                     size_t length,
                                     p11_array *objects);

bool             p11_persist_check  (p11_persist *persist,
                                     const char *filename,
                                     const unsigned char *data,
                                     size_t length);

bool             p11_persist_write  (p11_persist *persist,
                                     CK_ATTRIBUTE *object,
                                     p11_buffer *buf);

void             p11_persist_free   (p11_persist *persist);

bool             p11_persist_is_generated (const unsigned char *data,
					   size_t length);

#endif /* P11_PERSIST_H_ */
